#-----------------------------------------------------------------------
# File    : pcseval.py
# Contents: pattern counts evaluation functions
# Author  : Christian Borgelt
# History : 2013.05.09 file created from patred.py
#           2015.03.13 all functions replaced
#-----------------------------------------------------------------------
from math import log10, sqrt

oo = float('+inf')              # infinity as a constant

#-----------------------------------------------------------------------

def sigcount (pcs):
    sigs = 0                    # initalize the number of signatures
    zmax = max([z for z,c in pcs]+[0])
    for z in range(2,zmax+1):   # traverse the pattern sizes
        cmax = max([c for a,c in pcs if a == z]+[1])
        sigs += cmax-1          # sum the signatures per size
    return float(sigs)          # return the number of signatures

#-----------------------------------------------------------------------

def sumrho1 (pcs, psp):
    '''sumrho1 (pcs, psp)
Sum of rho(z,c) = (z-1)*c over the signatures.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the sum of rho(z,c) over the signatures'''
    n = float(sum([(z-1)*c for z,c in pcs]))
    d = float(sum([(z-1)*c for z,c in psp]))
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def sumrho2 (pcs, psp):
    '''sumrho2 (pcs, psp)
Sum of rho(z,c) = (z-1)*(c-1) over the signatures.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the sum of rho(z,c) over the signatures'''
    n = float(sum([(z-1)*(c-1) for z,c in pcs]))
    d = float(sum([(z-1)*(c-1) for z,c in psp]))
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def sumrho3 (pcs, psp):
    '''sumrho3 (pcs, psp)
Sum of rho(z,c) = (z-1)*c over non-excluded signatures
relative to number of signatures in surrogates.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the sum of rho(z,c) over the signatures'''
    n = float(sum([(s[0]-1)*s[1] for s in pcs if s not in psp]))
    d = sigcount(psp)
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def sumrho4 (pcs, psp):
    '''sumrho4 (pcs, psp)
Sum of rho(z,c) = (z-1)*(c-1) over non-excluded signatures
relative to number of signatures in surrogates.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the sum of rho(z,c) over the signatures'''
    n = float(sum([(s[0]-1)*(s[1]-1) for s in pcs if s not in psp]))
    d = sigcount(psp)
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def maxrho1 (pcs, psp):
    '''maxrho1 (pcs, psp)
Maximum of rho(z,c) = (z-1)*c over the signatures.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the maximum of rho(z,c) over the signatures'''
    n = float(max([(z-1)*c for z,c in pcs]+[0]))
    d = float(max([(z-1)*c for z,c in psp]+[0]))
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def maxrho2 (pcs, psp):
    '''maxrho2 (pcs, psp)
Maximum of rho(z,c) = (z-1)*(c-1) over the signatures.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the maximum of rho(z,c) over the signatures'''
    n = float(max([(z-1)*(c-1) for z,c in pcs]+[0]))
    d = float(max([(z-1)*(c-1) for z,c in psp]+[0]))
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def maxrho3 (pcs, psp):
    '''maxrho3 (pcs, psp)
Maximum of rho(z,c) = (z-1)*c over non-excluded signatures
relative to number of signatures in surrogates.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the maximum of rho(z,c) over the signatures'''
    n = float(max([(s[0]-1)*s[1] for s in pcs if s not in psp]+[0]))
    d = sigcount(psp)
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

def maxrho4 (pcs, psp):
    '''maxrho4 (pcs, psp)
Maximum of rho(z,c) = (z-1)*c over non-excluded signatures
relative to number of signatures in surrogates.
pcs     pattern counts in actual data as a dictionary
        mapping pairs (size, support) to a pattern count
psp     pattern spectrum derived from surrogate data
        as a set of pairs (size, support)
returns the maximum of rho(z,c) over the signatures'''
    n = float(max([(s[0]-1)*(s[1]-1) for s in pcs if s not in psp]+[0]))
    d = sigcount(psp)
    return n/d if d > 0 else 0.0 if n <= 0 else oo

#-----------------------------------------------------------------------

pcsfns = [sumrho1, sumrho2, sumrho3, sumrho4,
          maxrho1, maxrho2, maxrho3, maxrho4]

#-----------------------------------------------------------------------

def getpcsfn (i):
    return pcsfns[i] if i >= 0 else len(pcsfns)
